﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using GeoAPI.Geometries;
using HeuristicLab.Persistence.Core;
using HeuristicLab.Persistence.Interfaces;
using SharpMap.Data;
using SharpMap.Data.Providers;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;

namespace HeuristicLab.BioBoost.Persistence {

  public class GeometryFeatureProviderSerializer : ICompositeSerializer {

    #region ICompositeSerializer Members

    public int Priority { get { return 200; } }

    public bool CanSerialize(Type type) {
      return type == typeof (GeometryFeatureProvider);
    }

    public object CreateInstance(Type type, IEnumerable<Tag> metaInfo) {
      var table = new FeatureDataTable();
      foreach (var tag in metaInfo)
        table.Columns.Add(tag.Name, (Type)tag.Value);
      return new GeometryFeatureProvider(table);
    }

    public IEnumerable<Tag> CreateMetaInfo(object obj) {
      var gfp = obj as GeometryFeatureProvider;
      return from DataColumn col in gfp.Features.Columns select new Tag(col.ColumnName, col.DataType);
    }

    public IEnumerable<Tag> Decompose(object obj) {
      var gfp = obj as GeometryFeatureProvider;
      foreach (FeatureDataRow row in gfp.Features.Rows) {
        yield return new Tag(row.ItemArray);
        yield return new Tag(row.Geometry);
      }
    }

    public string JustifyRejection(Type type) {
      return "Not a GeometryFeatureProvider";
    }

    public void Populate(object instance, IEnumerable<Tag> tags, Type type) {
      var gfp = instance as GeometryFeatureProvider;
      var it = tags.GetEnumerator();
      while (it.MoveNext()) {
        var row = (FeatureDataRow) gfp.Features.LoadDataRow((object[]) it.Current.Value, LoadOption.OverwriteChanges);
        if (it.MoveNext())
          row.Geometry = (IGeometry) it.Current.Value;
        else
          throw new PersistenceException("invalid data format for GeometryDataProvider: interleaved pairs of features and geometries expected");
      }
    }

    #endregion
  }
}
